<?php if (!defined('PmWiki')) exit();

/* trail2menu.php Copyright 2004 Niall Durham (niall@webprose.cc)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program converts a wiki trail into an expanding side menu.
 */

// These variables contain HTML markup for the images that go in front
// of the various menu items.  I think the names suck.  If you have any
// suggestions, I'm all ears.
SDV($TrailMenuNoChildrenImg,     '');
SDV($TrailMenuExpandedImg,       '');
SDV($TrailMenuUnexpandedImg,     '');
SDV($TrailMenuCompressMidDepths, true);
SDV($TrailMenuDumpAllPages,      true);

// A slightly different breadcrumb trail implementation (modeled on
// MakeTrailPath() in trails.php).  It differs only in output.
//
// I don't want the root of the path to be the actual wiki trail page
// itself.  With this function, I can specify the trail prefix via 
// $path.
function BreadCrumbTrail($trail, $path='')
{
    global $pagename, $TrailPathSep;

    $tree = ReadTrail($trail);
    for ($i = 0; $i < count($tree); $i++) {
        if ($tree[$i]['name'] == $pagename) {
            while ($tree[$i]['depth'] > 0) {
                if ($crumb) $crumb = $TrailPathSep.$crumb; 
                $crumb = $tree[$i]['link'].$crumb;
                $i     = $tree[$i]['parent'];
            }
            if ($path) $crumb = $path.$TrailPathSep.$crumb;
            return "<span class=\"wikitrail\">$crumb</span>";
        }
    }
    return "<span class=\"wikitrail\">$path</span>";
}

// Print out a lot of diagnostic information about trail/site map. 
function DumpTrail($trail)
{
    global $WikiDir;
    global $TrailMenuDumpAllPages;

    $tree = ReadTrail($trail);

    $pageref = array();
    for ($i = 0; $i < count($tree); $i++) {
        $n = $tree[$i]['name'];
        if ( !isset($pageref[$n]) ) { 
            $pageref[$n]           = array();
            $pageref[$n]['links']  = array();
            $pageref[$n]['exists'] = false; // assume the worst
        }
        array_push($pageref[$n]['links'], $i);
        if (PageExists($n)) $pageref[$n]['exists'] = true;
    }
    ksort($pageref);
    
    $output  = "<h1>DumpTrail($trail)</h1>\n\n";
    $output .= "<h2>All Pages In Trail:</h2>\n";
    $cnt = 0;
    foreach ($pageref as $name => $data) {
        $output .= "$name - " . count($data['links']) . ' link(s)';
        if (!$data['exists']) $output .= ', %red%does not exist%%'; 
        $output .= "<br />\n";
        $cnt++;
    }
    $output .= "<strong>$cnt total unique page(s) in $trail</strong><br \>\n";

    // Pages that have more than 1 link to them.
    $output .= "\n<h2>More Than One Link:</h2>\n";
    reset($pageref);
    $cnt = 0;
    foreach ($pageref as $name => $data) {
        if (count($data['links']) > 1) {
            $output .= "$name<br />\n"; 
            $cnt++;
        }
    }
    $output .= "<strong>$cnt pages with more than one link in $trail</strong><br \>\n";

    // Pages that don't exist.
    $output .= "\n<h2>Pages In Trail That Do Not Exist:</h2>\n";
    reset($pageref);
    $cnt = 0;
    foreach ($pageref as $name => $data) {
        if (!$data['exists']) { 
            $output .= "$name<br />\n"; 
            $cnt++;
        }
    }
    $output .= "<strong>$cnt pages in $trail that do not exist</strong><br \>\n";

    // Pages that exist, but aren't linked in the trail.
    if ($TrailMenuDumpAllPages) {
        // Put all valid Wiki page filenames into an array.
        $allfiles = array();
        if ( is_dir($WikiDir) && $dirh = opendir($WikiDir) ) {
            while ( ($file = readdir($dirh)) !== false ) {
                if ( preg_match('/^(\w+\.\w+)$/', $file, $matches) ) 
                    $allfiles[] = $file;
            }
            closedir($dirh);
        }
        ksort($allfiles);

        $output .= "\n<h2>Existing Pages Not Linked In Trail:</h2>\n";
        $cnt = 0;
        foreach ($allfiles as $f) {
            if ( isset($pageref[$f]) ) continue;
            $output .= "$f<br />\n";
            $cnt++;
        }
        $output .= "<strong>$cnt pages not linked in $trail</strong><br \>\n";
    }

    return $output;
}

// Convert a Wiki Trail page (think site map) into an expanding 
// navigation menu.
//
// This approach is modeled after Pat's traversal algorithm (far more 
// efficient than the one I came up with).  I've modified the process 
// significantly to produce the output I want.
function Trail2Menu($trail)
{
    global $pagename;
    global $TrailMenuNoChildrenImg, $TrailMenuExpandedImg, 
           $TrailMenuUnexpandedImg, $TrailMenuCompressMidDepths;

    $tree = ReadTrail($trail);

    // Find current page in tree.
    for ($i = 0; $i < count($tree); $i++) {
        if ($tree[$i]['name'] == $pagename) { 
            $nidx = $i; 
            break; 
        }
    }

    $path = array();                     // current page to topmost ancestor
    while ($nidx) {
        $path[] = $nidx;
        $nidx = $tree[$nidx]['parent'];
        $min_depth = $tree[$nidx]['depth'];
    }
    $min_depth = $min_depth ? $min_depth : 1;

    // Walk the tree once and make note of all the nodes we want to 
    // display in the menu.  Also figure out which of those nodes have
    // children.
    $nodelist = array();                // indexes of nodes to put in menu
    for ($i = 0; $i < count($tree); $i++) {
        $p = $tree[$i]['parent'];
        if (in_array($p, $path) || $tree[$i]['depth'] == 1) {
            $tree[$i]['children'] = false;
            $nodelist[] = $i;
        }

        if (in_array($p, $nodelist)) $tree[$p]['children'] = true; 
    }

    // Determine maximum depth shown when displaying current node in 
    // tree.
    $i = $path[0];
    if ($tree[$i]['children'] == true) {
        $max_depth = $tree[$i]['depth'] + 1;
    } else {
        $max_depth = $tree[$i]['depth'];
    }

    // Print out links.
    $old_depth = 0;
    $output = '';
    foreach ($nodelist as $i) {
        // Take care of any changes in depth.  
        //
        // Assumes depth only changes by +1/-1 per step.
        $d = $tree[$i]['depth'];
        if ($d > $old_depth) { 
            $output .= "<ul class=\"depth$d\">\n";
            if ($old_depth+1 != $d) { } // TODO: warning ?
            $old_depth = $d;
        } elseif ($d < $old_depth) {
            while ($d < $old_depth) { 
                $output .= "</ul>\n</li>\n"; 
                $old_depth--; 
            }
        }

        // Don't print this link if this node is not one of the 
        // endpoints in the parent path.
        if (    $TrailMenuCompressMidDepths 
             && !in_array($i, $path) 
             && $d > $min_depth && $d < $max_depth ) continue;

        // Print out this link.
        if ($tree[$i]['children']) {    // nodes w/ children
            if (in_array($i, $path)) {  // showing all children 
                $output .= "<li class=\"expanded\">";
                if ($TrailMenuExpandedImg) { // markup img w/ link
                    $output .= Keep(FmtPageName("<a href='\$PageUrl'>$TrailMenuExpandedImg</a>", $tree[$i]['name']));
                }
                // Don't close that <li>!  Well, at least not yet. ;-)
                $output .= $tree[$i]['link'] . "\n"; 
            } else {                    // has children, but none shown
                $output .= "<li class=\"unexpanded\">"; 
                if ($TrailMenuUnexpandedImg) {
                    $output .= Keep(FmtPageName("<a href='\$PageUrl'>$TrailMenuUnexpandedImg</a>", $tree[$i]['name']));
                }
                $output .= $tree[$i]['link'] . "</li>\n";
            }
        } else {                        // no children
            $output .= "<li>"; 
            if ($TrailMenuNoChildrenImg) {
                $output .= Keep(FmtPageName("<a href='\$PageUrl'>$TrailMenuNoChildrenImg</a>", $tree[$i]['name']));
            }
            $output .= $tree[$i]['link'] . "</li>\n";
        }
    }
    
    while ($old_depth >= 1) { 
        $output .= ($old_depth > 2) ? "</ul>\n</li>\n" : "</ul>\n";  
        $old_depth--; 
    }

    return $output;
}

?>
